<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;

class GozoFeriasController extends Crud {

    protected $GozoFeriasService;
    protected $dataSourceName = 'getFerias';
    protected $dataSourceName1 = 'getGozoFerias';
    protected $entityManager;
    protected $nmArquivoPlanilhaF = 'Ferias';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\FPA\Services\GozoFeriasService $GozoFeriasService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->GozoFeriasService = $GozoFeriasService;
        parent::__construct($dataSourceManager);
    }

    public function getFerias(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getGozoFerias(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setFerias(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_ferias";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFERIAS', $row)) {
                    if ($row['NRFERIAS'] === '') {
                       $row['NRFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FERIAS', 1, 12, true); 
                    }
                } else {
                    $row['NRFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FERIAS', 1, 12, true); 
                }
                $row["QTDIAGOZOFERIAS"] = isset($row["QTDIASFERIAS"])?$row["QTDIASFERIAS"]:null;
                $row["DTAVISOFERIAS"]  = isset($row["DTAVISAOPREVIO"])?$row["DTAVISAOPREVIO"]:null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setGozoFerias(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_gozoferias";
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                // Bloqueio férias
                $vinculoRescindido = $this->GozoFeriasService->getRescisaoVinculo($row['NRVINCULOM']);
                
                if ($vinculoRescindido) {
                    return $response->addMessage(new DTO\Response\Message('O vínculo está rescindido!', DTO\Response\Message::TYPE_WARNING));
                }
                // End
                
                //verifica se há algum evento S-2230 em andamento ou sem um S-3000 concluído para possibilitar a alteração do dado
                $mensagem = $this->GozoFeriasService->verificaPossibilidadeAlteracao($row['NRGOZOFERIAS'], 'editar');
                
                if ($mensagem === true){
                    if (array_key_exists('NRGOZOFERIAS', $row)) {
                        if ($row['NRGOZOFERIAS'] === '') {
                            $row['NRGOZOFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GOZOFERIAS', 1, 12, true); 
                        }
                    } else {
                        $row['NRGOZOFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GOZOFERIAS', 1, 12, true); 
                    }
                } else {
                    if($mensagem instanceof \Zeedhi\Framework\DTO\Response\Message) {
                        $response->addMessage($mensagem);
                        return $response;
                    }
                }
            }
            $row['QTDIASFERIAS'] = isset($row['DIASFERIAS'])?$row['DIASFERIAS']:null;
            $row['DTAVISAOPREVIO'] = isset($row['DTAVISAOPREVIO'])?$row['DTAVISAOPREVIO']:null;

            
            $this->GozoFeriasService->validaGozoFerias($row);
            $avisaGozoAfastados = $this->GozoFeriasService->avisaGozoAfastados($row);
            if($avisaGozoAfastados){
                return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::evaluateMessage(Messages::HCM01000_FPA_GOZOFERIAS_VINCPOSSUIPERIODO_SITUFUNC_ERROR,$avisaGozoAfastados), DTO\Response\Message::TYPE_WARNING));
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteFerias(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->GozoFeriasService->deleteGozoFerias($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteGozoFerias(DTO\Request\DataSet $request, DTO\Response $response){
        //verifica se há algum evento S-2230 em andamento ou sem um S-3000 concluído para possibilitar a alteração do dado
        $row = $request->getDataSet()->getRows()[0];
        $mensagem = $this->GozoFeriasService->verificaPossibilidadeAlteracao($row['NRGOZOFERIAS'], 'excluir');
        
        $resultado = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy(array('nrorg' => $row['NRORG'],
        'nrvinculom' => $row['NRVINCULOM'], 'nrgozoferias' => $row['NRGOZOFERIAS'], 'nreventom' => 3030));

        if($resultado){
            $message = 'Operação bloqueada! Foram encontrados lançamentos de férias associados ao gozo';
            $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING);
            $response->addMessage($message);
            return;
        }
        
        if ($mensagem === true){
            $this->dataSourceName = "fpa_gozoferias";
            parent::delete($request, $response);
        } else {
            if($mensagem instanceof \Zeedhi\Framework\DTO\Response\Message) {
                $response->addMessage($mensagem);
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function baixarPlanilhaFerias(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $file = $this->GozoFeriasService->baixarPlanilhaFerias(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->nmArquivoPlanilhaF);
                
    
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaFerias(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->GozoFeriasService->importarPlanilhaFerias($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
} 