<?php
namespace HCM\FPA\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class TabelaFaixaRepository {

    protected  $connection;
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }
    
    /**
     * Essa função é responsavel por preparar e executar a query relativa a atualização de
     * um conjunto de faixas de um registro folha
     */
    public function reajustaValores($NRTABELAFOLHA, $DTINIVIGENCIA, $porcentagemReajuste) {
      $query = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_FAIXATABELA 
               SET VRAPLICARESULT = (VRAPLICARESULT + VRAPLICARESULT*$porcentagemReajuste)
            WHERE NRTABELAFOLHA = :NRTABELAFOLHA
        ");

      $query->execute(
          array(
              "NRTABELAFOLHA" => $NRTABELAFOLHA
          )
      );
    }
    
    /**
     * Função cujo propósito é obter todas as faixas de um dado registro da 
     * Tabela Folha
     */
    public function obterFaixas($row){
        $query = $this->entityManager->getConnection()->prepare(
           "SELECT
                FT.*,
                    TF.NMTABELAFOLHA 
                FROM
                    FPA_FAIXATABELA FT 
                INNER JOIN
                    FPA_TABELAFOLHA TF 
                        ON FT.NRTABELAFOLHA = TF.NRTABELAFOLHA 
                        AND FT.DTINIVIGENCIA IN(
                            SELECT
                                MAX (DTINIVIGENCIA) 
                        FROM
                            FPA_FAIXATABELA 
                        WHERE
                            DTINIVIGENCIA <= :DTINIVIGENCIA 
                            AND NRTABELAFOLHA = :NRTABELAFOLHA
                    ) 
                WHERE
                    FT.NRTABELAFOLHA = :NRTABELAFOLHA 
                ORDER BY
                    FT.DTINIVIGENCIA DESC,
                    FT.VRLIMITSUPFAIXA"
        );
        $query->execute([
            "NRTABELAFOLHA" => $row['oldNRTABELAFOLHA'], 
            "DTINIVIGENCIA" => $row['oldDTINIVIGENCIA']
        ]);
        return $query->fetchAll();
    }
    
    /**
     * Essa função, é executadoa logo após 'replicarFolha', cujo objetivo é replicar
     * também as faixas do registro da tabela folha replicado
     */
    public function replicarFaixas($auditoria, $row, $faixas){
        
        $query =  $this->entityManager->getConnection()->prepare(
            "INSERT INTO FPA_FAIXATABELA
                (
                    NRTABELAFOLHA, NRFAIXATABFOLHA, DTINIVIGENCIA, VRLIMITSUPFAIXA, VRPECALCULORESU,
                    VRMAXRESULTCALC, VRAPLICARESULT, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, 
                    NRORGULTATU, DTULTATU, CDOPERULTATU, NRORG
                )
            VALUES
                (
                    :NRTABELAFOLHA, :NRFAIXATABFOLHA, :DTINIVIGENCIA, :VRLIMITSUPFAIXA, :VRPECALCULORESU,
                    :VRMAXRESULTCALC, :VRAPLICARESULT, :NRORGINCLUSAO, :DTINCLUSAO, :CDOPERINCLUSAO, 
                    :NRORGULTATU, :DTULTATU, :CDOPERULTATU, :NRORG            
                )"
        );
       
        foreach($faixas as $faixa){
            // É gerado vários códigos, ao invés de gerar um único e incrementá-lo, para evitar
            // alguma inconsistência, no caso de alguém estiver duplicando as faixas e criando 
            // nova faixa simultâneamente
            $newNRFAIXATABFOLHA = \Geral\Util\NovoCodigo::selectMax('FPA_FAIXATABELA','NRFAIXATABFOLHA');
            $query->execute(
                [
                    "NRTABELAFOLHA" => $row['newNRTABELAFOLHA'], 
                    "NRFAIXATABFOLHA" => $newNRFAIXATABFOLHA, 
                    "DTINIVIGENCIA" => $row["newDTINIVIGENCIA"], 
                    "VRLIMITSUPFAIXA" => $faixa["VRLIMITSUPFAIXA"], 
                    "VRPECALCULORESU" => $faixa["VRPECALCULORESU"],
                    "VRMAXRESULTCALC" => $faixa["VRMAXRESULTCALC"], 
                    "VRAPLICARESULT" => $faixa["VRAPLICARESULT"], 
                    "NRORGINCLUSAO" => $auditoria["NRORGINCLUSAO"], 
                    "DTINCLUSAO" => $auditoria["DTINCLUSAO"], 
                    "CDOPERINCLUSAO" => $auditoria["CDOPERINCLUSAO"], 
                    "NRORGULTATU" => $auditoria["NRORGINCLUSAO"], 
                    "DTULTATU" => $auditoria["DTINCLUSAO"], 
                    "CDOPERULTATU" => $auditoria["CDOPERINCLUSAO"], 
                    "NRORG" => $auditoria["NRORG"]            
                ] 
            );
        }
    }
}