<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\ParametrizacaoSindicatoService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;

class ParametrizacaoSindicatoController {

    public function __construct (HCMParameters $hcmParameters, ParametrizacaoSindicatoService $parametrizacaoSindicatoService){
        $this->hcmParameters = $hcmParameters;
        $this->parametrizacaoSindicatoService = $parametrizacaoSindicatoService;
    }
    
    public function importarSindicatoPadrao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $sindicato) {
                if($sindicato['NRORG'] == 0){
                    $this->parametrizacaoSindicatoService->importarSindicatoPadrao($sindicato);
                    $response->addNotification(new Notification("Parâmetro replicado com sucesso para a estrutura ".$sindicato['NMESTRUTURAH'], Notification::TYPE_SUCCESS));
                }else{
                    $response->addMessage(new Message("Não é possível importar sindicato que não seja de organização padrão", Message::TYPE_ERROR));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 