<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\PTO\Services\EscalaTrabalhoService;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\CrudService;

class EscalaTrabalhoController extends Crud {

    protected $dataSourceGetEscalaTrab   = 'getEscalaTrab';
    protected $dataSourceSetEscalaTrabM  = 'gpe_escalatrabm';
    protected $dataSourceSetEscalaTrabH  = 'gpe_escalatrabh';
    protected $dataSourceSetHorarioTurno = 'gpe_horarioturno';
    protected $dataSourceGetHorarioTurno = 'gpeHorarioTurno_getHorarioTurno';


    protected $parameterBagColumns = array('NRORG', 'NRORG_PADRAO', 'DTMESCOMPETENC', 'IDATIVO', 'NRESCALATRABM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EscalaTrabalhoService $escalaTrabalhoService,CrudService $crudService, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->escalaTrabalhoService = $escalaTrabalhoService;
        $this->crudService = $crudService;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getEscalaTrab (DTO\Request\Filter $request, DTO\Response $response) {
        $this->parameterBagColumns = array('NRORG', 'NRORG_PADRAO', 'DTMESCOMPETENC');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetEscalaTrab);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setEscalaTrab (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetEscalaTrabM;
        
        try{
            foreach($request->getDataSet()->getRows() as &$row) {

                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                    $row['NRESCALATRABM'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_ESCALATRABM', 1, 12, true);
                } else {
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                }
                
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                
                $this->escalaTrabalhoService->validaCamposVazios($row);
            }
            
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$this->crudService->save($this->dataSourceName, $rows);
			$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            
			$currentRow = current($request->getDataSet()->getRows());
            if(isset($currentRow['saveHistoryTable'])) {
                    
                    if (!$currentRow['__is_new']) {
                        if (!$this->checkHistory($response, $currentRow)) {
                            // is new history
                            $currentRow['__is_new'] = true;
                        }
                    }
                    
				$tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
				$newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
					return $element[$tableMasterPK];
				});
                
				$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
				$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
				$currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);
                
                // set history properties
                $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
                
                $newDataSet = new DataSet($this->dataSourceName, array($currentRow));
                $newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
                $request->getMethod(), $request->getRoutePath(), $request->getUserId());
				$rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
				$this->crudService->save($this->dataSourceName, $rows);
            } 
            $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setEscalaTrabH (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetEscalaTrabH;
        try{
            foreach($request->getDataSet()->getRows() as $row) {

                $this->escalaTrabalhoService->validaCamposVazios($row);

                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                    $row['NRESCALATRABH'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_ESCALATRABH', 1, 12, true);
                } else {
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y H:i:s');
                    $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                    $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                }
                

            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveHorarioDiaAuto (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetHorarioTurno;
        $aux = array();
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $aux = $row['NRSEQHORARIO'];
                $row['IDATIVO'] = 'S';
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                $row['IDDIAVARIAVEL'] = 'N';
                
                foreach($aux as $seq){
                    $row['NRSEQHORARIO'] = $seq;
                    $row['NRHORARIOTURNO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGINCLUSAO'], 'GPE_HORARIOTURNO', 1, 12, true);
                    parent::save($request, $response);
                }
            }
                
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEscalaTrab (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $existeTurno = false;
            
            $existeTurno = $this->escalaTrabalhoService->verificaExistenciaTurno(
                $row->getRow()['NRESCALATRABM'],
                $row->getRow()['NRORG']
            );
            
            if($existeTurno)
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM07009_PTO_REGISTRO_FILHO_ERROR, DTO\Response\Message::TYPE_WARNING));
            else{
                $dataset = $row->getRow();
                $this->escalaTrabalhoService->deleteEscala($dataset);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
        
    public function checkHistory($response, $row) {
        $condition = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->hcmParameters->getNrorgTrabalho()
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_master_pk']),
                "operator" => "=",
                "value" => $row[strtoupper($row['saveHistoryTable']['__table_master_pk'])]
            ),
            array(
                "columnName" => strtoupper($row['saveHistoryTable']['__table_competenc']),
                "operator" => "=",
                "value" => $this->hcmParameters->getDtCompetencia()
            )
        );
        
        $filterCriteria = new FilterCriteria($row['saveHistoryTable']['__table_history_name'], $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array());
       
        $this->crudService->find($response, $newFilterCriteria);
        if (empty($response->getDataSets()[0]->getRows())) 
            return false;
        else 
            return true;
        
    }
    
    public function saveHorarioDiaAutoCiclo (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows()[0] as $row) {
                $row['IDATIVO'] = 'S';
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                $row['DTINCLUSAO'] = DateUtil::getDataAtual();
                $row['DTULTATU'] = DateUtil::getDataAtual();
                $row['IDDIAVARIAVEL'] = 'N';
                $row['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                $row['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
                $row['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                $row['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                $row['NRHORARIOTURNO'] = \HCM\Util\NovoCodigo::geraCodigo( $this->hcmParameters->getNrorgTrabalho(), 'GPE_HORARIOTURNO', 1, 12, true);
                
                $this->escalaTrabalhoService->saveHorarioDiaAutoCiclo($row);
            }
            $response->addNotification(new Notification("Cadastro automático realizado com sucesso!", Notification::TYPE_SUCCESS));
                
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}