<?php
/**
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 05/11/12
 * Time: 14:37
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\FechamentoDia;


use \Estoque\Entities\Diainvefili;
use \Estoque\Entities\Inverotalote;
use \Estoque\Entities\Inverota;
use \Estoque\Logic\FechamentoDia\Util;
use \Estoque\Util\Parametros;

use \Geral\Entities\Curvaabc;
use \Geral\Entities\Filial;
use \Geral\Entities\Invfiscfil;
use \Geral\Entities\Localestoque;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;
use \Geral\Entities\Servico;
use \Geral\Util\DateUtil;

use \MerendaPlanejamento\Entities\Prevconspme;

use \Setforms2\Runtime\Helper\Procedure\Param;
use \Setforms2\Runtime\Helper\Procedure;

class Inventario
{


    CONST FORMATEDATE_EN = 'y-MM-dd';


    /**
     * Calcula o percentual da curva do produto.
     *
     * @param $curva
     *
     * @return float|int|null
     */
    protected function percentualCurvaABC($curva){
        $result = 0;
        switch ($curva){
            case 'A':
                $result = (Parametros::getPercentualCurvaA() / 2);
                break;
            case 'B':
                $result = Parametros::getPercentualCurvaB();
                break;
            case 'C':
                $result = Parametros::getPercentualCurvaC();
                break;
            case 'T':
                $result = 100;
                break;
        }

        return $result;
    }


    /**
     * Calcula o inventario do estoque de acordo com a curva.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $utilLote
     * @param $dataAtual
     * @param $operador
     * @param $realizaInventDia
     * @param $geraInventRotaPosicao
     * @param $utilInventRotaRequisicao
     * @param $inventRealizado
     * @param $tipoInvent
     * @param $percentual
     *
     * @return bool
     */
    public static function calculoInventarioEstoque(
        $nrorg, $filial, $dtProc, $utilLote, $dataAtual, $operador, $realizaInventDia, $geraInventRotaPosicao,
        $utilInventRotaRequisicao, $inventRealizado, $tipoInvent, $percentual
    ){
        /** @var $localestoqueRepository \Geral\Repositories\Localestoque */
        $localestoqueRepository = Localestoque::getRepository();
        $dados = $localestoqueRepository->consultaAlmoxarifadoLocalizacaoPorFilial($nrorg, $filial);


        if ($dados) {
            /** @var $dado \Geral\Entities\Localestoque */
            foreach ($dados as $dado) {
                $cdAlmox = $dado->getAlmoxarife()->getCdalmoxarife();
                $cdLocal = $dado->getCdlocalestoq();

                if ($geraInventRotaPosicao  <> 'S'
                    AND $utilInventRotaRequisicao <> 'S'
                    AND $inventRealizado <> 'S'
                ) {
                    switch($tipoInvent){
                        case 'G':        // InventarioCurvaABC de acordo com a utilização do lote.
                            self::inventarioCurvaABC(
                                $nrorg, $filial, $dtProc, $utilLote, $cdAlmox, $cdLocal, $dataAtual, $operador
                            );
                            break;
                        case 'T':        // InventarioGeral de acordo com a utilização do lote.
                            self::inventarioGeral(
                                $nrorg, $filial, $dtProc, $utilLote, $cdAlmox, $cdLocal, $dataAtual, $operador);
                            break;
                        case 'N':        // InventarioNormal.
                            self::inventarioNormal(
                                $nrorg, $filial, $dtProc, $utilLote, $cdAlmox, $cdLocal,
                                $dataAtual, $operador, $realizaInventDia, $geraInventRotaPosicao,
                                $utilInventRotaRequisicao, $inventRealizado, $tipoInvent, $percentual
                            );
                            break;
                        default:
                            break;
                    }
                } else {
                    if ($geraInventRotaPosicao === 'S') {
                        self::inventarioPosicaoEstoque($nrorg, $filial, $dtProc, $utilLote, $dataAtual, $operador, $cdAlmox, $cdLocal);
                    } else if ($utilInventRotaRequisicao === 'S') {
                        if ($dados) {
                            self::inventarioRequisicao(
                                $nrorg, $filial, null, $dataAtual, $dtProc, $cdAlmox, $cdLocal, $utilLote, $realizaInventDia
                            );
                        }
                    }
                }
            }
        }
        return true;
    }


    /**
     * Exclui inventarios rotativos (usando ou não lote).
     *
     * @param $nrorg
     * @param $filial
     * @param $utilLote
     * @param $dtProc
     *
     * @return bool
     */
    public function excluiProdutosInventRota($nrorg, $filial, $utilLote, $dtProc){
        $params = array($nrorg, $filial,);                                                                                  // custo alto

        $produtosFilial = Prodfili::getObjectsByWhere(
            'NRORG = ? AND CDFILIAL = ?',
            $params
        );

        if ($produtosFilial) {
            if ($utilLote === 'S') {
                /** @var $produto \Geral\Entities\Prodfili */
                foreach ($produtosFilial as $produto) {
                    $param = array($filial, $nrorg, $dtProc, $produto['CDALMOXARIFE'],
                                   $produto['CDLOCALESTOQ'], $produto['CDPRODUTO'],);

                    /** @var $inverotalote Inverotalote */
                    $inverotalote = Inverotalote::getObjectByWhere(
                        'CDFILIAL = ? AND NRORG = ? AND DTINVELOTE = ? AND CDALMOXARIFE = ? AND CDLOCALESTOQ = ? AND
                        CDPRODUTO = ?',
                        $param
                    );

                    $inverotalote->delete();
                }
            } else {
                /** @var $produto \Geral\Entities\Prodfili */
                foreach ($produtosFilial as $produto) {
                    /** @var $inverota Inverota */
                    $inverota = Inverota::getObject(
                        array(
                            'cdfilial' => $filial,
                            'nrorg' => $nrorg,
                            'dtinvelote' => $dtProc,
                            'cdalmoxarife' => $produto->getLocalestoq()->getAlmoxarife()->getCdalmoxarife(),
                            'cdlocalestoq' => $produto->getLocalestoq()->getCdlocalestoq(),
                            'cdproduto' => $produto->getProduto()->getCdproduto(),
                        )
                    );

                    $inverota->delete();
                }
            }
        }

         return true;
    }


    /**
     * Insere inventarios rotativos de acordo com os produtos da CurvaABC, usando ou não lote.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $utilLote
     * @param $almox
     * @param $local
     * @param $dataAtual
     * @param $operador
     *
     * @return bool
     */
    public function inventarioCurvaABC(
            $nrorg, $filial, $dtProc, $utilLote, $almox, $local, $dataAtual, $operador
    ){
        if ($utilLote === 'S') {
            /** @var $inverotaloteRepository \Estoque\Repositories\Inverotalote */
            $inverotaloteRepository = Inverotalote::getRepository();
            $inverotaloteRepository->insereInventarioRotativoLotePosicao(
                    $nrorg, $filial, $dtProc, $almox, $local, $dataAtual, $operador
            );

        } else {
            $params = array(
                $filial,
                $nrorg,
                $almox,
                $local,
            );

            $listaProdutos = Curvaabc::getObjectsByWhere(
                'CDFILIAL = ? AND NRORG = ? AND CDALMOXARIFE = ? AND CDLOCALESTOQ = ?',
                $params,
                'IDCURVAABC, CDPRODUTO'
            );

            if ($listaProdutos) {
                $inverota = new Inverota();
                /** @var $produto \Geral\Entities\Curvaabc */
                foreach ($listaProdutos as $produto) {
                    $inverota->insereInventarioRotativo(
                            $nrorg, $filial, $dtProc, 0, 0, $almox, $local,
                            $produto->getProduto()->getCdproduto(), $dataAtual, $operador
                    );
                }
            }
        }

        return true;
    }


    /**
     * Insere inventarios rotativos de acordo com os produtos da filial com controle de estoque e
     * posição do estoque no dia.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $utilLote
     * @param $almox
     * @param $local
     * @param $dataAtual
     * @param $operador
     *
     * @return bool
     */
    public function inventarioGeral(
            $nrorg, $filial, $dtProc, $utilLote, $almox, $local, $dataAtual, $operador
    ){
        if ($utilLote === 'S') {
            /** @var $inverotaloteRepository \Estoque\Repositories\Inverotalote */
            $inverotaloteRepository = Inverotalote::getRepository();
            $inverotaloteRepository->insereInventarioRotativoLoteDtInventario(
                $nrorg, $filial, $dtProc, $dataAtual, $operador);

        } else {
            /** @var $prodfiliRepository  \Geral\Repositories\Prodfili*/
            $prodfiliRepository = \Geral\Entities\Prodfili::getRepository();
            $produtos = $prodfiliRepository->consultaProdutosFilial($filial, $nrorg);

            /** @var $produto \Geral\Entities\Produto */
            foreach ($produtos as $produto) {
                $inverota = new Inverota();
                $inverota->insereInventarioRotativo(
                        $nrorg, $filial, $dtProc, 0, 0, $almox, $local,
                        $produto->getCdproduto(), $dataAtual, $operador
                );
            }
        }

        return true;
    }


    public function inventarioNormal(
            $nrorg, $filial, $dtProc, $utilLote, $almox, $local, $dataAtual, $operador, $inventRotativoDia,
            $geraInventRotaPosicao = null, $utilInventRotaRequisicao = null, $percentual = null
    ){
        $insercao = 0;

        if ($geraInventRotaPosicao <> 'S' AND $utilInventRotaRequisicao <> 'S') {
            $params = array($filial, $nrorg, $almox, $local,);

            /** @var $listaProdutosCurvaABC Curvaabc */
            $listaProdutosCurvaABC = Curvaabc::getObjectsByWhere(
                'CDFILIAL = ? AND NRORG = ? AND CDALMOXARIFE = ? AND CDLOCALESTOQ = ?',
                $params,
                'IDCURVAABC, CDPRODUTO'
            );

            if ($listaProdutosCurvaABC === false) {
                return true;
            }

            if ($percentual === 100) {
                self::executaInventariado(
                        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, 'T', 'T',
                        $listaProdutosCurvaABC, $dataAtual, $operador
                );

                $params = array($filial, $nrorg, $listaProdutosCurvaABC->getIdcurvaabc(), $almox, $local);

                /** @var $objCurva \Geral\Entities\Curvaabc */
                $objCurva = Curvaabc::getObjectByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND (IDCURVAABC = ? OR \'T\' = \'T\') AND
                    CDALMOXARIFE = ? AND CDLOCALESTOQ = ?',
                    $params
                );

                $objCurva->atualizaMarcaInventario('N');
            } else {
                // Curva A
                self::executaInventariado(
                        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "A", "T",
                        $listaProdutosCurvaABC, $dataAtual, $operador
                );

                self::executaInventariado(
                        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "A", "N",
                        $listaProdutosCurvaABC, $dataAtual, $operador
                );

                // Curva B
                self::executaInventariado(
                        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "B", "N",
                        $listaProdutosCurvaABC, $dataAtual, $operador
                );

                if ($insercao > 0) {
                    self::executaInventariado(
                            $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "B", "T",
                            $listaProdutosCurvaABC, $dataAtual, $operador
                    );
                }

                // Curva C
                self::executaInventariado(
                        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "C", "N",
                        $listaProdutosCurvaABC, $dataAtual, $operador
                );
                if ($insercao > 0) {
                    self::executaInventariado(
                            $filial, $nrorg, $dtProc, $utilLote, $almox, $local, $insercao, "C", "T",
                            $listaProdutosCurvaABC, $dataAtual, $operador
                    );
                }
            }
        } else {
            if ($geraInventRotaPosicao === 'S') {
                self::inventarioPosicaoEstoque($nrorg, $filial, $dtProc, $utilLote, $dataAtual, $operador, $almox, $local);
            } else if ($utilInventRotaRequisicao === 'S') {
                self::inventarioRequisicao(
                    $nrorg, $filial, null, $dataAtual, $dtProc, $almox, $local, $utilLote, $inventRotativoDia
                );

                self::inventarioRequisicao(
                    $nrorg, $filial, null, $dataAtual, $dtProc, $almox, $local, $utilLote, $inventRotativoDia
                );
            }
        }

        return true;
    }


    protected function executaInventariado(
        $filial, $nrorg, $dtProc, $utilLote, $almox, $local, &$insercao, $curva, $marca,
        $listaProdutosCurvaABC, $dataAtual, $operador
    ){
        $idCurvaABC = 0;
        $qtdProdutos = count($listaProdutosCurvaABC);

        /** @var $produto Curvaabc */
        foreach ($listaProdutosCurvaABC as $produto) {
            $idCurvaABC = $produto->getIdcurvaabc();
            break;
        }

        /** @var $curvaABCRepository \Geral\Repositories\Curvaabc */
        $curvaABCRepository = Curvaabc::getRepository();
        $qtdProdutosInvent = $curvaABCRepository->consultaQtdProdutosInventariados(
            $filial, $nrorg, $almox, $local, $idCurvaABC
        );


        if ($marca === 'T' AND $insercao > 0) {
            $prodCurva = $insercao;
            $bool = true;
        } else {
            $percentualInvent = Parametros::getPercentualInventarioRotativo();
            $percentualCurvaABC = self::percentualCurvaABC($curva);
            $prodCurva = round($qtdProdutos * (($percentualInvent * $percentualCurvaABC / 100) / 100), 0);
            $insercao = $insercao + $prodCurva;
            $prodCurva = $insercao;
            $bool = false;
        }

        if ($prodCurva >= $qtdProdutosInvent AND $bool === false) {
            self::insereInventarioRotativo(
                $nrorg, $filial, $insercao, $marca, $dtProc, $utilLote, $almox, $local,
                $listaProdutosCurvaABC, $dataAtual, $operador
            );
        } else {
            self::verificaProdutoInventariado(
                $filial, $nrorg, $insercao, $dtProc, $utilLote, $almox, $local, $curva,
                $marca, $prodCurva, $dataAtual, $operador
            );
        }

        return true;
    }


    public function verificaProdutoInventariado(
        $filial, $nrorg, &$insercao, $dtProc, $utilLote, $almox, $local, $curva,
        $marca, $prodCurva, $dataAtual, $operador
    ){
        $listaProdutosCurvaABCMarca = Curvaabc::getObject(
            array(
                'cdfilial' => $filial,
                'nrorg' => $nrorg,
                'cdalmoxarife' => $almox,
                'cdlocalestoq' => $local,
                'idcurvaabc' => $curva,
                'idmarcinve' => 'N',
            )
        );

        if ($listaProdutosCurvaABCMarca) {
            $totalProd = count($listaProdutosCurvaABCMarca);
            $count = 1;
            $numRand = rand(0, $totalProd);

            while ($count <= $prodCurva) {
                /** @var $dadosCurva \Geral\Entities\Curvaabc */
                $dadosCurva = $listaProdutosCurvaABCMarca[$numRand];
                $params = array($filial, $nrorg, $dtProc, $dadosCurva->getProduto()->getCdproduto());

                if ($utilLote === 'S') {
                    $produto = Inverotalote::getObjectByWhere(
                        'CDFILIAL = ? AND NRORG = ? AND DTINVEROTA = ? AND CDPRODUTO = ?',
                        $params
                    );
                } else {
                    $produto = Inverota::getObjectByWhere(
                        'CDFILIAL = ? AND NRORG = ? AND DTINVEROTA = ? AND CDPRODUTO = ?',
                        $params
                    );
                }

                if ($produto){
                    $dadosCurva = array($dadosCurva);
                    self::insereInventarioRotativo(
                        $nrorg, $filial, $insercao, $marca, $dtProc, $utilLote, $almox, $local,
                        $dadosCurva, $dataAtual, $operador
                    );

                    break;
                } else {
                    if ($count === $prodCurva) {
                        $numRand--;
                        if ($numRand < 0) {
                            break;
                        }
                    } else {
                        $numRand++;
                        $count++;
                    }
                }
            }
        }

        return true;
    }


    /**
     * Insere inventario rotativo.
     *
     * @param $nrorg
     * @param $filial
     * @param $insercao
     * @param $marca
     * @param $dtProc
     * @param $utilLote
     * @param $almox
     * @param $local
     * @param $listaProdutos
     * @param $dataAtual
     * @param $operador
     *
     * @return bool
     */
    protected function insereInventarioRotativo(
        $nrorg, $filial, &$insercao, $marca, $dtProc, $utilLote, $almox, $local, $listaProdutos, $dataAtual, $operador
    ){
        if ($utilLote === 'S') {
            $data = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtProc));
            $dtPosicao = $data->format(DateUtil::FORMATO_BRASILEIRO);

            /** @var $prodFiliRespository \Geral\Repositories\Prodfili */
            $prodFiliRespository = Prodfili::getRepository();

            /** @var $produto Curvaabc */
            foreach ($listaProdutos as $produto) {
                if ($marca === 'T') {
                    $id = $produto->getIdcurvaabc();
                    $cdProduto = $produto->getProduto()->getCdproduto();

                    $objCurva = Curvaabc::getObject(
                        array(
                            'cdfilial' => $filial,
                            'nrorg' => $nrorg,
                            'idcurvaabc' => $id,
                            'cdproduto' => $cdProduto,
                            'cdalmoxarife' => $almox,
                            'cdlocalestoq' => $local,
                        )
                    );

                    /** @var $objCurva \Geral\Entities\Curvaabc */
                    $objCurva->atualizaMarcaInventario('S');
                }

                $dadosProd = $prodFiliRespository->consultaDadosProdutoFilial(
                    $nrorg, $filial, $dtPosicao, $produto->getProduto()->getCdproduto(), $almox, $local
                );

                /** @var $inverotaloteRepository \Estoque\Repositories\Inverotalote */
                $inverotaloteRepository = Inverotalote::getRepository();
                $inverotaloteRepository->insereInventarioRotativoLoteLocal(
                    $nrorg, $filial, $dtProc, $almox, $local, $dadosProd['NRLOTEESTQ'],
                    $dadosProd['NRSUBLOTE'], $dadosProd['CDPRODUTO'], $dadosProd['QTESTOQDIA'],
                    $dadosProd['QTESTOQDIA'], $dadosProd['IDCNTRESTOQ'], $operador, $dataAtual
                );

                $insercao--;
            }
        } else {
            /** @var $produto Curvaabc */
            foreach ($listaProdutos as $produto) {
                if ($marca === 'T') {
                    $id = $produto->getIdcurvaabc();
                    $cdProduto = $produto->getProduto()->getCdproduto();

                    $objCurva = Curvaabc::getObject(
                        array(
                            'cdfilial' => $filial,
                            'nrorg' => $nrorg,
                            'idcurvaabc' => $id,
                            'cdproduto' => $cdProduto,
                            'cdalmoxarife' => $almox,
                            'cdlocalestoq' => $local,
                        )
                    );

                    /** @var $objCurva \Geral\Entities\Curvaabc */
                    $objCurva->atualizaMarcaInventario('S');
                }


                /** @var $inverotaRepository \Estoque\Repositories\Inverota */
                $inverotaRepository = Inverota::getRepository();
                $inverotaRepository->insereInventarioRotativoLocal(
                    $nrorg, $filial, $dtProc, 0, 0, $almox, $local, $produto->getProduto()->getCdproduto(),
                    $operador, $dataAtual
                );

                $insercao--;
            }
        }

        return true;
    }


    public function inventarioNormalProxy(
        $nrorg, $filial, $dtProc, $utilLote, $almox, $local, $dataAtual, $operador, $inventRotativoDia
    ){
        /*$geraInventRotaPosicao, $utilInventRotaRequisicao, $percentual*/
        return self::inventarioNormal(
            $nrorg, $filial, $dtProc, $utilLote, $almox, $local, $dataAtual, $operador, $inventRotativoDia
        );
    }


    public function calculoInventarioFiscalMes($nrorg, $filial, $dtProc, $operador, $dataAtual){
        $Util = new Util();
        /** @var $data \Estoque\Logic\FechamentoDia\Util */
        $data = $Util->primeiroUltimoDiaMes($dtProc);
        // $dataInventario = new \DateTime($data->ultimoDiaMes);
        $dataInventario = DateUtil::getDataDeString($data->ultimoDiaMes);

        /* DELETA ANTIGOS */
        $param = array($filial, $nrorg, $dataInventario);
        /** @var $invFiscFil Invfiscfil */
        $invFiscFil = Invfiscfil::getObjectsByWhere(
            'CDFILIAL = ? AND NRORG = ? AND DTINVFISCFIL = ?',
            $param
        );

        if ($invFiscFil) {
            /** @var $inventario Invfiscfil */
            foreach ($invFiscFil as $inventario) {
                $inventario->delete();
            }
        }

        /* INSERE NOVOS DE ACORDO COM A POSICAO DO ESTOQUE NO DIA */
        $invFiscFil = new Invfiscfil();
        $invFiscFil->insereInventFiscal($nrorg, $filial, $data->ultimoDiaMes, $operador, $dataAtual);
        return true;
    }


    public function inventarioPosicaoEstoque($nrorg, $filial, $dtProc, $utilLote, $dtAtual, $operador, $cdAlmox, $cdLocal){
        $data = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtProc));
        $dtProxProc = $data->format(DateUtil::FORMATO_BRASILEIRO);

        /** @var $prodFiliRepository \Geral\Repositories\Prodfili */
        $prodFiliRepository = Prodfili::getRepository();
        $produtos = $prodFiliRepository->consultaProdutosDtPosicao($nrorg, $filial, $dtProxProc, $cdAlmox, $cdLocal);

        if ($produtos) {
            if ($utilLote === 'S') {
                /** @var $inverotaLoteRepository \Estoque\Repositories\Inverotalote */
                $inverotaLoteRepository = Inverotalote::getRepository();

                $data = new \TkDateTime($dtProc);
                $dtPosAnt = $data->subDay(1)->toString(self::FORMATEDATE_EN);

                foreach ($produtos as $produto) {
                    $inverotaLoteRepository->insereInventarioRotativoOK(
                        $filial, $nrorg, $dtAtual, $operador, $dtProc, $dtPosAnt,
                        $produto['CDALMOXARIFE'], $produto['CDLOCALESTOQ']
                    );
                }
            } else {
                /** @var $inverotaRepository \Estoque\Repositories\Inverota */
                $inverotaRepository = Inverota::getRepository();

                foreach ($produtos as $produto) {
                    $inverotaRepository->insereInventarioRotativoLocal(
                        $nrorg, $filial, $dtProxProc, $produto['QTESTOQDIA'], $produto['QTESTOQDIA'],
                        $produto['CDALMOXARIFE'], $produto['CDLOCALESTOQ'], $produto['CDPRODUTO'],
                        $operador, $dtAtual
                    );
                }
            }
        }

        return true;
    }


    public function inventarioRequisicao(
        $nrorg, $filial, $objFilial = null, $dataAtual, $dtProc,
        $cdAlmox, $cdLocal, $utilLote, $inventRotativoDia
    ) {

        if (empty($objFilial) === true ) {
            $objFilial = Filial::getObject(array(
                'cdfilial' => $filial,
                'nrorg'  => $nrorg,
            ));
        }

        /** @var $objFilial Filial */
        $centralMerenda = $objFilial->getCdfilcentpme();

        if ($centralMerenda) {
            $tipoCard     = '';
            $listaServico = array();

            /** @var $servicoRepository \Geral\Repositories\Servico */
            $servicoRepository = Servico::getRepository();
            $dadosServico = $servicoRepository->consultaListaServicoFilial($nrorg, $filial, $centralMerenda);
            if ($dadosServico) {
                foreach($dadosServico as $servico) {
                    $listaServico[] = $servico['NMSERVICO'];
                }

                $servico = implode('_', $listaServico);

                $diaInvent = Diainvefili::consultaDiaInventariado($nrorg, $filial);
                if($diaInvent) {
                    $dataFechamento = DateUtil::getDataDeString($dtProc);
                    $mesAnoFecham   = $dataFechamento->format("/m/Y");
                    $dtInvent       = $diaInvent.$mesAnoFecham;
                } else {
                    $dtInvent       = $dtProc;
                }

                if ($inventRotativoDia === Paramfilial::UTILIZA_INVENTARIO_ROTATIVO_DIARIO){
                    $dtInicio = $dtProc;
                } else if ($dtInvent < $dtProc) {
                    $dtInicio = $dtInvent;
                } else {
                    $data     = DateUtil::getDataDeString($dtInvent);
                    $dtInicio = DateUtil::subtraiIntervalo($data, 1, DateUtil::MES);
                    $dtInicio = DateUtil::adicionaIntervalo($dtInicio, 1, DateUtil::DIA);
                    $dtInicio = $dtInicio->format(DateUtil::FORMATO_BRASILEIRO);
                }

                $dtFinal    = $dtProc;
                $acrescimo  = 0;
                $prodInicio = '0';
                $prodFinal  = 'z';
                $operador   = '';

                $codigo = \Geral\Util\NovoCodigo::geraCodigo($nrorg, 'CONSPLAN'.$centralMerenda);

                $arredonda = 'N';
                $procedure = new Procedure("PREVISAO_CONSUMO_PME");

                $procedure->addParam(new Param(
                    "P_CDFILCENTPME",
                    Param::PARAM_INPUT,
                    $centralMerenda,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_CDTIPOCARD",
                    Param::PARAM_INPUT,
                    $tipoCard,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_CDFILIAL",
                    Param::PARAM_INPUT,
                    $filial,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_CDSERVFILI",
                    Param::PARAM_INPUT,
                    $servico,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_DTINICIAL",
                    Param::PARAM_INPUT,
                    $dtInicio,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_DTFINAL",
                    Param::PARAM_INPUT,
                    $dtFinal,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_PEACRESCIMO",
                    Param::PARAM_INPUT,
                    (int)$acrescimo,
                    Param::PARAM_TYPE_INT
                ));
                $procedure->addParam(new Param(
                    "P_CDPRODUTOINI",
                    Param::PARAM_INPUT,
                    $prodInicio,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_CDPRODUTOFIN",
                    Param::PARAM_INPUT_OUTPUT,
                    $prodFinal,
                    Param::PARAM_TYPE_STR,
                    1
                ));
                $procedure->addParam(new Param(
                    "P_CDOPERADOR",
                    Param::PARAM_INPUT,
                    $operador,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_DTHRCALCPC",
                    Param::PARAM_INPUT,
                    $dataAtual,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_NRCONSPLAN",
                    Param::PARAM_INPUT,
                    $codigo,
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_IDARREDOND",
                    Param::PARAM_INPUT,
                    $arredonda,
                    Param::PARAM_TYPE_STR
                ));

                $procedure->call();

                $Inverota     = new Inverota();
                $InverotaLote = new Inverotalote();

                /** @var $prodfiliRepository \Geral\Repositories\Prodfili */
                $prodfiliRepository = Prodfili::getRepository();

                /** @var $prevconsumoRepository \MerendaPlanejamento\Repositories\Prevconspme */
                $prevconsumoRepository = Prevconspme::getRepository();
                $dadosPrevConsumo = $prevconsumoRepository->consultaPrevisaoConsumoMerenda (
                    $nrorg, $filial, $dtInicio, $dtFinal
                );

                if ($dadosPrevConsumo) {
                    foreach ($dadosPrevConsumo as $dados) {
                        $produtoPrev = $prevconsumoRepository->consultaProdutosPrevisao(
                            $nrorg, $filial, $dados['CDFILCENTPME'], $dados['CDSERVFILI'], $dados['DTPREVCONPME']
                        );

                        foreach ($produtoPrev as $produtos) {
                            if ($utilLote === 'S') {
                                $lote = $prodfiliRepository->consultaDadosProdutoFilial(
                                    $nrorg, $filial, $dtProc, $produtos['CDPRODUTO'], $cdAlmox, $cdLocal
                                );

                                $InverotaLote->insereInventarioRotativoLote(
                                    $nrorg, $filial, $dtProc, $cdAlmox, $cdLocal, $lote['NRLOTEESTQ'], $lote['NRSUBLOTE'],
                                    $lote['CDPRODUTO'], 0, 0, $lote['IDCNTRESTOQ'], $dataAtual, $operador
                                );
                            } else {
                                $Inverota->insereInventarioRotativo(
                                    $nrorg, $filial, $dtProc, 0, 0, $cdAlmox, $cdLocal,
                                    $produtos['CDPRODUTO'], $dataAtual, $operador
                                );
                            }
                        }
                    }
                }
            }
            return true;
        } else {
            return false;
        }
    }


}